@@ -1,9 +1,9 @@
 /*
- * $Id: Config.cs 283 2008-07-16 21:26:07Z meebey $
+ * $Id: Config.cs 464 2008-12-28 22:01:41Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Engine/Config/Config.cs $
- * $Rev: 283 $
+ * $Rev: 464 $
  * $Author: meebey $
- * $Date: 2008-07-16 23:26:07 +0200 (Wed, 16 Jul 2008) $
+ * $Date: 2008-12-28 23:01:41 +0100 (Sun, 28 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -201,7 +201,7 @@ namespace Smuxi.Engine
             prefix = "Engine/Users/DEFAULT/Interface/Notebook/";
             Get(prefix+"TimestampFormat", "HH:mm");
             Get(prefix+"TabPosition", "top");
-            Get(prefix+"BufferLines", 100);
+            Get(prefix+"BufferLines", 500);
             Get(prefix+"EngineBufferLines", 100);
             Get(prefix+"StripColors", false);
             Get(prefix+"StripFormattings", false);
@@ -209,7 +209,7 @@ namespace Smuxi.Engine
             prefix = "Engine/Users/DEFAULT/Interface/Notebook/Tab/";
             Get(prefix+"NoActivityColor", "#000000");
             Get(prefix+"ActivityColor",   "#0080FF");
-            Get(prefix+"ModeColor",       "#2020C0");
+            Get(prefix+"EventColor",      "#2020C0");
             Get(prefix+"HighlightColor",  "#E80000");
             
             prefix = "Engine/Users/DEFAULT/Interface/Notebook/Channel/";
@@ -223,13 +223,17 @@ namespace Smuxi.Engine
             Get(prefix+"FontFamily", String.Empty);
             Get(prefix+"FontStyle",  String.Empty);
             Get(prefix+"FontSize",   0);
+            Get(prefix+"WrapMode",   "Char");
             
             prefix = "Engine/Users/DEFAULT/Interface/Entry/";
             Get(prefix+"CompletionCharacter", ":");
             Get(prefix+"CommandCharacter", "/");
             Get(prefix+"BashStyleCompletion", false);
             Get(prefix+"CommandHistorySize", 30);
-            
+
+            prefix = "Engine/Users/DEFAULT/Interface/Notification/";
+            Get(prefix+"NotificationAreaIconMode", "Minimized");
+
             prefix = "Engine/Users/DEFAULT/Sound/";
             Get(prefix+"BeepOnHighlight", false);
             
@@ -250,7 +254,11 @@ namespace Smuxi.Engine
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "OFTC");
             Get(prefix + "OnStartupConnect", true);
-            Get(prefix + "OnConnectCommands", new string[] { "/join #smuxi" });
+            Get(prefix + "OnConnectCommands",
+                new string[] {
+                    "/join #smuxi",
+                }
+            );
             
             prefix = "Engine/Users/DEFAULT/Servers/IRC/irc.gimp.org/";
             Get(prefix + "Hostname", "irc.gimp.org");
@@ -330,7 +338,7 @@ namespace Smuxi.Engine
                     }
                 } catch (Exception ex) {
 #if LOG4NET
-                    _Logger.Warn("Load(): error getting realname from gecos", ex);
+                    _Logger.Warn("Load(): error getting realname from gecos (ignoring)", ex);
 #endif
                 }
                 if (String.IsNullOrEmpty(realname)) {
@@ -360,10 +368,12 @@ namespace Smuxi.Engine
                 LoadUserEntry(user, "Interface/Chat/FontFamily", null);
                 LoadUserEntry(user, "Interface/Chat/FontStyle", null);
                 LoadUserEntry(user, "Interface/Chat/FontSize", null);
+                LoadUserEntry(user, "Interface/Chat/WrapMode", null);
                 LoadUserEntry(user, "Interface/Entry/CompletionCharacter", null);
                 LoadUserEntry(user, "Interface/Entry/CommandCharacter", null);
                 LoadUserEntry(user, "Interface/Entry/BashStyleCompletion", null);
                 LoadUserEntry(user, "Interface/Entry/CommandHistorySize", null);
+                LoadUserEntry(user, "Interface/Notification/NotificationAreaIconMode", null);
                 
                 LoadUserEntry(user, "Sound/BeepOnHighlight", null);
                 
@@ -409,8 +419,8 @@ namespace Smuxi.Engine
                     LoadEntry(sprefix+"Port", null);
                     LoadEntry(sprefix+"Network", null);
                     LoadEntry(sprefix+"Encoding", null);
-                    LoadEntry(sprefix+"Username", null);
-                    LoadEntry(sprefix+"Password", null);
+                    LoadEntry(sprefix+"Username", String.Empty);
+                    LoadEntry(sprefix+"Password", String.Empty);
                     LoadEntry(sprefix+"OnStartupConnect", false);
                     string[] commands = GetList(sprefix + "OnConnectCommands");
                     if (commands == null) {
