@@ -1,9 +1,9 @@
 /*
- * $Id: FrontendManager.cs 378 2008-08-24 00:26:35Z meebey $
+ * $Id: FrontendManager.cs 479 2009-01-01 15:32:22Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Engine/FrontendManager.cs $
- * $Rev: 378 $
+ * $Rev: 479 $
  * $Author: meebey $
- * $Date: 2008-08-24 02:26:35 +0200 (Sun, 24 Aug 2008) $
+ * $Date: 2009-01-01 16:32:22 +0100 (Thu, 01 Jan 2009) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -36,17 +36,12 @@ namespace Smuxi.Engine
 {
     public delegate void SimpleDelegate(); 
     
-    // TODO: use generics for the queue
-    public class FrontendManager : PermanentRemoteObject, IFrontendUI
+    public class FrontendManager : PermanentRemoteObject, IFrontendUI, IDisposable
     {
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
         private static readonly string       _LibraryTextDomain = "smuxi-engine";
-        private int              _Version = 0;
-        // queue needs to be thread-safe for different protocol manager threads
-        private Queue            _Queue  = Queue.Synchronized(new Queue()); 
-        private Thread           _Thread;
         private Session          _Session;
         private IFrontendUI      _UI;
         private ChatModel        _CurrentChat;
@@ -55,10 +50,11 @@ namespace Smuxi.Engine
         private SimpleDelegate   _ConfigChangedDelegate;
         private bool             _IsFrontendSynced;
         private IList<ChatModel> _SyncedChats = new List<ChatModel>();
-
+        private TaskQueue        f_TaskQueue;
+        
         public int Version {
             get {
-                return _Version;
+                return 0;
             }
         }
         
@@ -97,7 +93,7 @@ namespace Smuxi.Engine
         
         public bool IsAlive {
             get {
-                return _Thread.IsAlive;
+                return  !f_TaskQueue.Disposed;
             }
         }
         
@@ -114,19 +110,37 @@ namespace Smuxi.Engine
             
             _Session = session;
             _UI = ui;
-            _Thread = new Thread(new ThreadStart(_Worker));
-            _Thread.IsBackground = true;
-            _Thread.Start();
+            f_TaskQueue = new TaskQueue("FrontendManager");
+            f_TaskQueue.ExceptionEvent += OnTaskQueueExceptionEvent;
+            f_TaskQueue.AbortedEvent   += OnTaskQueueAbortedEvent;
             
             // register event for config invalidation
             // BUG: when the frontend disconnects there are dangling methods registered!
             //_Session.Config.Changed += new EventHandler(_OnConfigChanged);
+        }
+        
+        ~FrontendManager()
+        {
+            Trace.Call();
             
-            // BUG: Session adds stuff to the queue but the frontend is not ready yet!
-            // The frontend must Sync() _first_!
-            // HACK: so this bug doesn't happen for now
-            // actually there is no other way, the frontend must tell us when he is ready to sync!
-            //Sync();
+            Dispose(false);
+        }
+        
+        public void Dispose()
+        {
+            Trace.Call();
+            
+            Dispose(true);
+            GC.SuppressFinalize(this);
+        }
+        
+        protected void Dispose(bool disposing)
+        {
+            Trace.Call(disposing);
+            
+            if (disposing) {
+                f_TaskQueue.Dispose();
+            }
         }
         
         public void Sync()
@@ -179,7 +193,7 @@ namespace Smuxi.Engine
                 } else {
                     pos = 0;
                 }
-                CurrentProtocolManager = (IProtocolManager) _Session.ProtocolManagers[pos];
+                CurrentProtocolManager = _Session.ProtocolManagers[pos];
             }
             
             UpdateNetworkStatus();
@@ -203,7 +217,9 @@ namespace Smuxi.Engine
         
         private void _AddChat(ChatModel chat)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.AddChat, chat));
+            f_TaskQueue.Queue(delegate {
+                _UI.AddChat(chat);
+            });
         }
         
         public void AddTextToChat(ChatModel chat, string text)
@@ -218,7 +234,9 @@ namespace Smuxi.Engine
         
         public void EnableChat(ChatModel chat)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.EnableChat, chat));
+            f_TaskQueue.Queue(delegate {
+                _UI.EnableChat(chat);
+            });
         }
         
         public void DisableChat(ChatModel chat)
@@ -226,19 +244,28 @@ namespace Smuxi.Engine
             if (_SyncedChats.Contains(chat)) {
                 _SyncedChats.Remove(chat);
             }
-            _Queue.Enqueue(new UICommandContainer(UICommand.DisableChat, chat));
+            f_TaskQueue.Queue(delegate {
+                _UI.DisableChat(chat);
+            });
         }
         
         public void AddMessageToChat(ChatModel chat, MessageModel msg)
         {
-            if (_SyncedChats.Contains(chat)) {
-                _AddMessageToChat(chat, msg);
+            if (!_SyncedChats.Contains(chat)) {
+#if LOG4NET
+                _Logger.Warn("AddMessageToChat(): chat: " + chat + " is not synced yet, ignoring call...");
+#endif
+                return;
             }
+            
+            _AddMessageToChat(chat, msg);
         }
         
         private void _AddMessageToChat(ChatModel chat, MessageModel msg)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.AddMessageToChat, chat, msg));
+            f_TaskQueue.Queue(delegate {
+                _UI.AddMessageToChat(chat, msg);
+            });
         }
         
         public void AddMessageToCurrentChat(MessageModel msg)
@@ -248,14 +275,9 @@ namespace Smuxi.Engine
         
         public void RemoveChat(ChatModel chat)
         {
-            if (_SyncedChats.Contains(chat)) {
-                _RemoveChat(chat);
-            }
-        }
-        
-        private void _RemoveChat(ChatModel chat)
-        {
-            _Queue.Enqueue(new UICommandContainer(UICommand.RemoveChat, chat));
+            f_TaskQueue.Queue(delegate {
+                _UI.RemoveChat(chat);
+            });
         }
         
         public void SyncChat(ChatModel chat)
@@ -267,152 +289,87 @@ namespace Smuxi.Engine
         
         private void _SyncChat(ChatModel chat)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.SyncChat, chat));
+            f_TaskQueue.Queue(delegate {
+                _UI.SyncChat(chat);
+            });
         }
         
         public void AddPersonToGroupChat(GroupChatModel groupChat, PersonModel person)
         {
-            if (_SyncedChats.Contains(groupChat)) {
-                _AddPersonToGroupChat(groupChat, person);
+            if (!_SyncedChats.Contains(groupChat)) {
+                return;
             }
+            
+            _AddPersonToGroupChat(groupChat, person);
         }
         
         private void _AddPersonToGroupChat(GroupChatModel groupChat, PersonModel person)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.AddPersonToGroupChat, groupChat, person));
+            f_TaskQueue.Queue(delegate {
+                _UI.AddPersonToGroupChat(groupChat, person);
+            });
         }
         
         public void UpdatePersonInGroupChat(GroupChatModel groupChat, PersonModel oldPerson, PersonModel newPerson)
         {
-            if (_SyncedChats.Contains(groupChat)) {
-                _UpdatePersonInGroupChat(groupChat, oldPerson, newPerson);
+            if (!_SyncedChats.Contains(groupChat)) {
+                return;
             }
+            
+            _UpdatePersonInGroupChat(groupChat, oldPerson, newPerson);
         }
         
         private void _UpdatePersonInGroupChat(GroupChatModel groupChat, PersonModel oldPerson, PersonModel newPerson)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.UpdatePersonInGroupChat, groupChat, oldPerson, newPerson));
+            f_TaskQueue.Queue(delegate {
+                _UI.UpdatePersonInGroupChat(groupChat, oldPerson, newPerson);
+            });
         }
     
         public void UpdateTopicInGroupChat(GroupChatModel groupChat, string topic)
         {
-            if (_SyncedChats.Contains(groupChat)) {
-                _UpdateTopicInGroupChat(groupChat, topic);
+            if (!_SyncedChats.Contains(groupChat)) {
+                return;
             }
+            
+            _UpdateTopicInGroupChat(groupChat, topic);
         }
         
         private void _UpdateTopicInGroupChat(GroupChatModel groupChat, string topic)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.UpdateTopicInGroupChat, groupChat, topic));
+            f_TaskQueue.Queue(delegate {
+                _UI.UpdateTopicInGroupChat(groupChat, topic);
+            });
         }
     
         public void RemovePersonFromGroupChat(GroupChatModel groupChat, PersonModel person)
         {
-            if (_SyncedChats.Contains(groupChat)) {
-                _RemovePersonFromGroupChat(groupChat, person);
+            if (!_SyncedChats.Contains(groupChat)) {
+                return;
             }
+            
+            _RemovePersonFromGroupChat(groupChat, person);
         }
         
         private void _RemovePersonFromGroupChat(GroupChatModel groupChat, PersonModel person)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.RemovePersonFromGroupChat, groupChat, person));
+            f_TaskQueue.Queue(delegate {
+                _UI.RemovePersonFromGroupChat(groupChat, person);
+            });
         }
         
         public void SetNetworkStatus(string status)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.SetNetworkStatus, status));
+            f_TaskQueue.Queue(delegate {
+                _UI.SetNetworkStatus(status);
+            });
         }
         
         public void SetStatus(string status)
         {
-            _Queue.Enqueue(new UICommandContainer(UICommand.SetStatus, status));
-        }
-        
-        private void _Worker()
-        {
-            while (true) {
-                if (_Queue.Count > 0) {
-                    try {
-                        UICommandContainer com = (UICommandContainer)_Queue.Dequeue();
-                        switch (com.Command) {
-                            case UICommand.AddChat:
-                                _UI.AddChat((ChatModel)com.Parameters[0]);
-                                break;
-                            case UICommand.RemoveChat:
-                                _UI.RemoveChat((ChatModel)com.Parameters[0]);
-                                break;
-                            case UICommand.EnableChat:
-                                _UI.EnableChat((ChatModel)com.Parameters[0]);
-                                break;
-                            case UICommand.DisableChat:
-                                _UI.DisableChat((ChatModel)com.Parameters[0]);
-                                break;
-                            case UICommand.SyncChat:
-                                _UI.SyncChat((ChatModel)com.Parameters[0]);
-                                break;
-                            case UICommand.AddMessageToChat:
-                                _UI.AddMessageToChat((ChatModel)com.Parameters[0],
-                                    (MessageModel)com.Parameters[1]);
-                                break;
-                            case UICommand.AddPersonToGroupChat:
-                                _UI.AddPersonToGroupChat((GroupChatModel)com.Parameters[0],
-                                    (PersonModel)com.Parameters[1]);
-                                break;
-                            case UICommand.UpdatePersonInGroupChat:
-                                _UI.UpdatePersonInGroupChat((GroupChatModel)com.Parameters[0],
-                                    (PersonModel)com.Parameters[1], (PersonModel)com.Parameters[2]);
-                                break;
-                            case UICommand.UpdateTopicInGroupChat:
-                                _UI.UpdateTopicInGroupChat((GroupChatModel)com.Parameters[0],
-                                    (string)com.Parameters[1]);
-                                break;
-                            case UICommand.RemovePersonFromGroupChat:
-                                _UI.RemovePersonFromGroupChat((GroupChatModel)com.Parameters[0],
-                                    (PersonModel)com.Parameters[1]);
-                                break;
-                            case UICommand.SetNetworkStatus:
-                                _UI.SetNetworkStatus((string)com.Parameters[0]);
-                                break;
-                            case UICommand.SetStatus:
-                                _UI.SetStatus((string)com.Parameters[0]);
-                                break;
-                            default:
-#if LOG4NET
-                                _Logger.Error("_Worker(): Unknown UICommand: "+com.Command);
-#endif
-                                break;
-                        }
-                    } catch (System.Runtime.Remoting.RemotingException e) {
-#if LOG4NET
-                        if (!_IsFrontendDisconnecting) {
-                            // we didn't expect this problem
-                            _Logger.Error("RemotingException in _Worker(), aborting FrontendManager thread...", e);
-                            _Logger.Error("Inner-Exception: ", e.InnerException);
-                        }
-#endif
-                        // TODO: setup a timer and wait up to 10 minutes to let
-                        // the frontend resume the session, after that timeout
-                        // clean it good
-                        break;
-                    } catch (Exception e) {
-#if LOG4NET
-                        _Logger.Error("Exception in _Worker(), aborting FrontendManager thread...", e);
-                        _Logger.Error("Inner-Exception: ", e.InnerException);
-#endif
-                        break;
-                    }
-                } else {
-                    // no better way?
-                    // wrap Queue and raise an event when Add() is called
-                    Thread.Sleep(10);
-                }
-            }
-            
-            // we can't rely on the UI object here, the connection is probably
-            // gone and doesn't come back
-            //_Session.DeregisterFrontendUI(_UI);
-            // thus we can deregister the hardway (using ourself)
-            _Session.DeregisterFrontendManager(this);
+            f_TaskQueue.Queue(delegate {
+                _UI.SetStatus(status);
+            });
         }
         
         private void _OnConfigChanged(object sender, EventArgs e)
@@ -433,6 +390,40 @@ namespace Smuxi.Engine
             }
         }
         
+        protected virtual void OnTaskQueueExceptionEvent(object sender, TaskQueueExceptionEventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            if (e.Exception is System.Runtime.Remoting.RemotingException) {
+#if LOG4NET
+                if (!_IsFrontendDisconnecting) {
+                    // we didn't expect this problem
+                    _Logger.Error("RemotingException in TaskQueue, aborting thread...", e.Exception);
+                    _Logger.Error("Inner-Exception: ", e.Exception.InnerException);
+                }
+#endif
+                // TODO: setup a timer and wait up to 10 minutes to let
+                // the frontend resume the session, after that timeout
+                // clean it good
+            } else {
+#if LOG4NET
+                _Logger.Error("Exception in TaskQueue, aborting thread...", e.Exception);
+                _Logger.Error("Inner-Exception: ", e.Exception.InnerException);
+#endif
+            }
+        }
+        
+        protected virtual void OnTaskQueueAbortedEvent(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            // we can't rely on the UI (proxy) object here, the connection is probably
+            // gone and doesn't come back
+            //_Session.DeregisterFrontendUI(_UI);
+            // thus we can deregister the hardway (by using our instance)
+            _Session.DeregisterFrontendManager(this);            
+        }
+        
         private static string _(string msg)
         {
             return LibraryCatalog.GetString(msg, _LibraryTextDomain);
