@@ -1,9 +1,9 @@
 /*
- * $Id: Session.cs 378 2008-08-24 00:26:35Z meebey $
+ * $Id: Session.cs 481 2009-01-01 17:12:36Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Engine/Session.cs $
- * $Rev: 378 $
+ * $Rev: 481 $
  * $Author: meebey $
- * $Date: 2008-08-24 02:26:35 +0200 (Sun, 24 Aug 2008) $
+ * $Date: 2009-01-01 18:12:36 +0100 (Thu, 01 Jan 2009) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -37,15 +37,16 @@ namespace Smuxi.Engine
     public class Session : PermanentRemoteObject, IFrontendUI 
     {
 #if LOG4NET
-        private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
+        private static readonly log4net.ILog f_Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
-        private static readonly string       _LibraryTextDomain = "smuxi-engine";
+        private static readonly string                _LibraryTextDomain = "smuxi-engine";
         private int                                   _Version = 0;
         private IDictionary<string, FrontendManager>  _FrontendManagers; 
         private IList<IProtocolManager>               _ProtocolManagers;
         private IList<ChatModel>                      _Chats;
         private SessionChatModel                      _SessionChat;
         private Config                                _Config;
+        private string                                _Username;
         private ProtocolManagerFactory                _ProtocolManagerFactory;
         private UserConfig                            _UserConfig;
         private bool                                  _OnStartupCommandsProcessed;
@@ -86,6 +87,12 @@ namespace Smuxi.Engine
             }
         }
         
+        public bool IsLocal {
+            get {
+                return _Username == "local";
+            }
+        }
+        
         public Session(Config config, ProtocolManagerFactory protocolManagerFactory,
                        string username)
         {
@@ -103,20 +110,15 @@ namespace Smuxi.Engine
             
             _Config = config;
             _ProtocolManagerFactory = protocolManagerFactory;
+            _Username = username;
             
             _FrontendManagers = new Dictionary<string, FrontendManager>();
             _ProtocolManagers = new List<IProtocolManager>();
-            _Chats = new List<ChatModel>();
             _UserConfig = new UserConfig(config, username);
+            _Chats = new List<ChatModel>();
             
             _SessionChat = new SessionChatModel("smuxi", "smuxi");
             _Chats.Add(_SessionChat);
-            
-            MessageModel msg = new MessageModel();
-            msg.MessageParts.Add(
-                new TextMessagePartModel(new TextColor(0xFF0000), null, false,
-                        true, false, _("Welcome to Smuxi")));
-            AddMessageToChat(_SessionChat, msg); 
         }
         
         public void RegisterFrontendUI(IFrontendUI ui)
@@ -127,12 +129,9 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("ui");
             }
             
-            string uri = RemotingServices.GetObjectUri((MarshalByRefObject)ui);
-            if (uri == null) {
-                uri = "local";
-            }
+            string uri = GetUri(ui);
 #if LOG4NET
-            _Logger.Debug("Registering UI with URI: "+uri);
+            f_Logger.Debug("Registering UI with URI: " + uri);
 #endif
             // add the FrontendManager to the hashtable with an unique .NET remoting identifier
             FrontendManager fm = new FrontendManager(this, ui);
@@ -141,13 +140,31 @@ namespace Smuxi.Engine
             // if this is the first frontend, we process OnStartupCommands
             if (!_OnStartupCommandsProcessed) {
                 _OnStartupCommandsProcessed = true;
-                ChatModel smuxiChat = GetChat("smuxi", ChatType.Protocol, null);
+                
+                MessageModel msg;
+                msg = new MessageModel();
+                msg.MessageParts.Add(
+                    new TextMessagePartModel(new TextColor(0xFF0000), null, false,
+                            true, false, _("Welcome to Smuxi")));
+                AddMessageToChat(_SessionChat, msg);
+                
+                msg = new MessageModel();
+                msg.MessageParts.Add(
+                    new TextMessagePartModel(null, null, false,
+                            true, false, _("Type /help to get a list of available commands.")));
+                AddMessageToChat(_SessionChat, msg);
+
+                msg = new MessageModel();
+                msg.MessageParts.Add(
+                    new TextMessagePartModel(null, null, false,
+                            true, false, _("After you have made a connection the list of available commands changes, just use /help again.")));
+                AddMessageToChat(_SessionChat, msg);
                 
                 foreach (string command in (string[])_UserConfig["OnStartupCommands"]) {
                     if (command.Length == 0) {
                         continue;
                     }
-                    CommandModel cd = new CommandModel(fm, smuxiChat,
+                    CommandModel cd = new CommandModel(fm, _SessionChat,
                         (string)_UserConfig["Interface/Entry/CommandCharacter"],
                         command);
                     bool handled;
@@ -184,7 +201,7 @@ namespace Smuxi.Engine
                     // if the connect command was correct, we should be able to get
                     // the chat model
                     if (protocolManager.Chat == null) {
-                        fm.AddTextToChat(smuxiChat, String.Format(_("Automatic connect to {0} failed!"), server.Hostname + ":" + server.Port));
+                        fm.AddTextToChat(_SessionChat, String.Format(_("Automatic connect to {0} failed!"), server.Hostname + ":" + server.Port));
                         continue;
                     }
                     
@@ -224,8 +241,15 @@ namespace Smuxi.Engine
                 }
             }
             if (key == null) {
-                throw new InvalidOperationException("Could not find key for frontend manager in _FrontendManagers.");
+#if LOG4NET
+                f_Logger.Debug("DeregisterFrontendManager(fm): could not find " +
+                               "frontend manager (probably already cleanly " +
+                               " deregistered), ignoring...");
+#endif
+                //throw new InvalidOperationException("Could not find key for frontend manager in _FrontendManagers.");
+                return;
             }
+            
             _FrontendManagers.Remove(key);
         }
         
@@ -237,12 +261,9 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("ui");
             }
             
-            string uri = RemotingServices.GetObjectUri((MarshalByRefObject)ui);
-            if (uri == null) {
-                uri = "local";
-            }
+            string uri = GetUri(ui);
 #if LOG4NET
-            _Logger.Debug("Deregistering UI with URI: "+uri);
+            f_Logger.Debug("Deregistering UI with URI: "+uri);
 #endif
             _FrontendManagers.Remove(uri);
         }
@@ -255,11 +276,29 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("ui");
             }
             
-            string uri = RemotingServices.GetObjectUri((MarshalByRefObject)ui);
-            if (uri == null) {
-                uri = "local";
+            return _FrontendManagers[GetUri(ui)];
+        }
+        
+        private string GetUri(IFrontendUI ui)
+        {
+            if (ui == null) {
+                throw new ArgumentNullException("ui");
             }
-            return (FrontendManager)_FrontendManagers[uri];
+            
+            if (IsLocal) {
+                return "local";
+            }
+            
+            return RemotingServices.GetObjectUri((MarshalByRefObject)ui);
+        }
+        
+        public static bool IsLocalFrontend(IFrontendUI ui)
+        {
+            if (ui == null) {
+                throw new ArgumentNullException("ui");
+            }
+            
+            return RemotingServices.GetObjectUri((MarshalByRefObject)ui) == null;
         }
         
         public ChatModel GetChat(string id, ChatType chatType, IProtocolManager networkManager)
@@ -345,14 +384,13 @@ namespace Smuxi.Engine
             cd.FrontendManager.AddMessageToChat(cd.Chat, msg);
             
             string[] help = {
-            "help",
-            "connect/server protocol [protocol-parameters]",
-            "disconnect",
-            "network list",
-            "network close [server]",
-            "network switch [server]",
-            "config (save|load)",
-            "quit [quitmessage]",
+                "help",
+                "connect/server protocol [protocol-parameters]",
+                "disconnect",
+                "network list",
+                "network close [server]",
+                "network switch [server]",
+                "config (save|load)",
             };
             
             foreach (string line in help) { 
@@ -441,9 +479,13 @@ namespace Smuxi.Engine
                         return;
                     }
                 }
-                fm.AddTextToCurrentChat("-!- " + String.Format(
-                                                    _("Disconnect failed, could not find server: {0}"),
-                                                    server));
+                fm.AddTextToCurrentChat(
+                    "-!- " +
+                    String.Format(
+                        _("Disconnect failed, could not find server: {0}"),
+                        server
+                    )
+                );
             } else {
                 fm.CurrentProtocolManager.Disconnect(fm);
                 _ProtocolManagers.Remove(fm.CurrentProtocolManager);
@@ -623,9 +665,6 @@ namespace Smuxi.Engine
             _Chats.Add(chat);
             foreach (FrontendManager fm in _FrontendManagers.Values) {
                 fm.AddChat(chat);
-                // BUG: race condition: the (group) chat isn't fully ready yet to be synced
-                // The ProtocolManager will tell us when the chat is ready to be synced
-                //fm.SyncChat(chat);
             }
         }
         
@@ -705,7 +744,7 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("msg");
             }
             
-            int buffer_lines = (int)UserConfig["Interface/Notebook/EngineBufferLines"];
+            int buffer_lines = (int) UserConfig["Interface/Notebook/EngineBufferLines"];
             if (buffer_lines > 0) {
                 chat.UnsafeMessages.Add(msg);
                 if (chat.UnsafeMessages.Count > buffer_lines) {
@@ -728,7 +767,7 @@ namespace Smuxi.Engine
             }
             
 #if LOG4NET
-            _Logger.Debug("AddPersonToGroupChat() groupChat.Name: "+groupChat.Name+" person.IdentityName: "+person.IdentityName);
+            f_Logger.Debug("AddPersonToGroupChat() groupChat.Name: "+groupChat.Name+" person.IdentityName: "+person.IdentityName);
 #endif
             foreach (FrontendManager fm in _FrontendManagers.Values) {
                 fm.AddPersonToGroupChat(groupChat, person);
@@ -748,7 +787,7 @@ namespace Smuxi.Engine
             }
             
 #if LOG4NET
-            _Logger.Debug("UpdatePersonInGroupChat()" +
+            f_Logger.Debug("UpdatePersonInGroupChat()" +
                           " groupChat.Name: " + groupChat.Name +
                           " oldPerson.IdentityName: " + oldPerson.IdentityName +
                           " newPerson.IdentityName: " + newPerson.IdentityName);
@@ -770,7 +809,7 @@ namespace Smuxi.Engine
             }
 
 #if LOG4NET
-            _Logger.Debug("UpdateTopicInGroupChat() groupChat.Name: " + groupChat.Name + " topic: " + topic);
+            f_Logger.Debug("UpdateTopicInGroupChat() groupChat.Name: " + groupChat.Name + " topic: " + topic);
 #endif
             groupChat.Topic = topic;
             foreach (FrontendManager fm in _FrontendManagers.Values) {
@@ -788,7 +827,7 @@ namespace Smuxi.Engine
             }
             
 #if LOG4NET
-            _Logger.Debug("RemovePersonFromGroupChat() groupChat.Name: " + groupChat.Name + " person.ID: "+person.ID);
+            f_Logger.Debug("RemovePersonFromGroupChat() groupChat.Name: " + groupChat.Name + " person.ID: "+person.ID);
 #endif
             groupChat.UnsafePersons.Remove(person.ID.ToLower());
             foreach (FrontendManager fm in _FrontendManagers.Values) {
@@ -803,7 +842,7 @@ namespace Smuxi.Engine
             }
             
 #if LOG4NET
-            _Logger.Debug("SetNetworkStatus() status: "+status);
+            f_Logger.Debug("SetNetworkStatus() status: "+status);
 #endif
             foreach (FrontendManager fm in _FrontendManagers.Values) {
                 fm.SetNetworkStatus(status);
@@ -817,7 +856,7 @@ namespace Smuxi.Engine
             }
             
 #if LOG4NET
-            _Logger.Debug("SetStatus() status: "+status);
+            f_Logger.Debug("SetStatus() status: "+status);
 #endif
             foreach (FrontendManager fm in _FrontendManagers.Values) {
                 fm.SetStatus(status);
@@ -829,6 +868,34 @@ namespace Smuxi.Engine
             return _ProtocolManagerFactory.GetProtocols();
         }
         
+        public void Connect(FrontendManager frontendManager, string protocol,
+                            string hostname, int port,
+                            string username, string password)
+        {
+            Trace.Call(frontendManager, protocol, hostname, port, username, "XXX");
+            
+            IProtocolManager protocolManager = _CreateProtocolManager(
+                frontendManager,
+                protocol
+            );
+            if (protocolManager == null) {
+                throw new ApplicationException(
+                    String.Format(
+                        _("No protocol manager found for the protocol: {0}"),
+                        protocol
+                    )
+                );
+            }
+            
+            _ProtocolManagers.Add(protocolManager);
+            // only pass non-empty passwords to Connect()
+            if (String.IsNullOrEmpty(password)) {
+                password = null;
+            }
+            protocolManager.Connect(frontendManager, hostname, port,
+                                    username, password);
+        }
+        
         private IProtocolManager _CreateProtocolManager(FrontendManager fm, string protocol)
         {
             ProtocolManagerInfoModel info = _ProtocolManagerFactory.GetProtocolManagerInfoByAlias(protocol); 
