@@ -1,9 +1,9 @@
 /*
- * $Id: AboutDialog.cs 299 2008-07-18 01:38:49Z meebey $
+ * $Id: AboutDialog.cs 433 2008-10-11 15:42:54Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/AboutDialog.cs $
- * $Rev: 299 $
+ * $Rev: 433 $
  * $Author: meebey $
- * $Date: 2008-07-18 03:38:49 +0200 (Fri, 18 Jul 2008) $
+ * $Date: 2008-10-11 17:42:54 +0200 (Sat, 11 Oct 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -33,15 +33,31 @@ namespace Smuxi.Frontend.Gnome
 {
     public class AboutDialog : Gtk.AboutDialog
     {
-        public AboutDialog()
+        public AboutDialog(Gtk.Window parent)
         {
+            if (parent == null) {
+                throw new ArgumentNullException("parent");
+            }
+            
+            TransientFor = parent;
             Name = Frontend.Name;
             Version = "\n Frontend: " + Frontend.UIName + " " + Frontend.Version +
                       "\n Engine: " + Frontend.EngineVersion;
-            Copyright = "2005-2008 (C) Mirco Bauer <meebey@meebey.net>";
+            Copyright = "Copyright © 2005-2008 Mirco Bauer <meebey@meebey.net>";
             Authors = new string[] {"Mirco Bauer <meebey@meebey.net>"};
-            TranslatorCredits = "German - Mirco Bauer <meebey@meebey.net>";
+            TranslatorCredits = _("German") + " - Mirco Bauer <meebey@meebey.net>\n" +
+                                _("Spanish") + " - Juan Miguel Carrero <streinleght@gmail.com>\n" +
+                                _("British English") + " - Ryan Smith-Evans <Kimera.Kimera@gmail.com>\n" +
+                                _("French") + " - Clement BOURGEOIS <moonpyk@gmail.com>\n" +
+                                _("Italian") + " - David Paleino <d.paleino@gmail.com>";
             Logo = new Gdk.Pixbuf(null, "about.png");
+            Website = "http://www.smuxi.org/";
+            WebsiteLabel = _("Smuxi Website");
+        }
+        
+        private static string _(string msg)
+        {
+            return Mono.Unix.Catalog.GetString(msg);
         }
     }
 }
