@@ -1,9 +1,9 @@
 /*
- * $Id: ChatView.cs 374 2008-08-22 19:29:46Z meebey $
+ * $Id: ChatView.cs 458 2008-12-28 21:13:04Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Chats/ChatView.cs $
- * $Rev: 374 $
+ * $Rev: 458 $
  * $Author: meebey $
- * $Date: 2008-08-22 21:29:46 +0200 (Fri, 22 Aug 2008) $
+ * $Date: 2008-12-28 22:13:04 +0100 (Sun, 28 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -54,6 +54,8 @@ namespace Smuxi.Frontend.Gnome
         private   bool               _AtUrlTag;
         private   ChatModel          _ChatModel;
         private   bool               _HasHighlight;
+        private   bool               _HasActivity;
+        private   bool               _HasEvent;
         private   Gtk.TextMark       _EndMark;
         private   Gtk.Menu           _TabMenu;
         private   Gtk.Label          _TabLabel;
@@ -91,7 +93,12 @@ namespace Smuxi.Frontend.Gnome
         }
         
         public bool HasActivity {
+            get {
+                return _HasActivity;
+            }
             set {
+                _HasActivity = value;
+                
                 if (HasHighlight) {
                     // don't show activity if there is a highlight active
                     return;
@@ -107,7 +114,30 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
-        public virtual bool HasSelection {
+        public bool HasEvent {
+            get {
+                return _HasEvent;
+            }
+            set {
+                if (HasHighlight) {
+                    return;
+                }
+                if (HasActivity) {
+                    return;
+                }
+                
+                if (!value) {
+                    // clear event with "no activity"
+                    HasActivity = false;
+                    return;
+                }
+                
+                string color = (string) Frontend.UserConfig["Interface/Notebook/Tab/EventColor"];
+                _TabLabel.Markup = String.Format("<span foreground=\"{0}\">{1}</span>", color, _Name);
+            }
+        }
+        
+        public bool HasTextViewSelection {
             get {
 #if GTK_SHARP_2_10
                 return _OutputTextView.Buffer.HasSelection;
@@ -119,6 +149,12 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
+        public virtual bool HasSelection {
+            get {
+                return HasTextViewSelection;
+            }
+        }
+        
         public virtual new bool HasFocus {
             get {
                 return base.HasFocus || _OutputTextView.HasFocus;
@@ -220,7 +256,6 @@ namespace Smuxi.Frontend.Gnome
             tv.Editable = false;
             //tv.CursorVisible = false;
             tv.CursorVisible = true;
-            //tv.WrapMode = Gtk.WrapMode.WordChar;
             tv.WrapMode = Gtk.WrapMode.Char;
             tv.Buffer.Changed += new EventHandler(_OnTextBufferChanged);
             tv.MotionNotifyEvent += new Gtk.MotionNotifyEventHandler(_OnMotionNotifyEvent);
@@ -399,7 +434,7 @@ namespace Smuxi.Frontend.Gnome
                     // HACK: the engine should set a color for us!
                     Gtk.TextTag urlTag = _OutputTextTagTable.Lookup("url");
                     Gdk.Color urlColor = urlTag.ForegroundGdk;
-                    Console.WriteLine("urlColor: " + urlColor);
+                    //Console.WriteLine("urlColor: " + urlColor);
                     TextColor urlTextColor = ColorTools.GetTextColor(urlColor);
                     urlTextColor = ColorTools.GetBestTextColor(urlTextColor, bgTextColor);
                     //Console.WriteLine("GetBestTextColor({0}, {1}): {2}",  urlColor, bgTextColor, urlTextColor);
@@ -451,6 +486,9 @@ namespace Smuxi.Frontend.Gnome
             // HACK: out of scope?
             if (hasHighlight && !Frontend.MainWindow.HasToplevelFocus) {
                 Frontend.MainWindow.UrgencyHint = true;
+#if GTK_SHARP_2_10
+                Frontend.StatusIcon.Blinking = true;
+#endif
                 if (Frontend.UserConfig["Sound/BeepOnHighlight"] != null &&
                     (bool)Frontend.UserConfig["Sound/BeepOnHighlight"]) {
                     Frontend.MainWindow.Display.Beep();
@@ -459,11 +497,19 @@ namespace Smuxi.Frontend.Gnome
             
             // HACK: out of scope?
             if (Frontend.MainWindow.Notebook.CurrentChatView != this) {
-                if (hasHighlight) {
+                if (hasHighlight &&
+                    _ChatModel.LastSeenHighlight < msg.TimeStamp) {
                     HasHighlight = true;
                 }
                 
-                HasActivity = true;
+                switch (msg.MessageType) {
+                    case MessageType.Normal:
+                        HasActivity = true;
+                        break;
+                    case MessageType.Event:
+                        HasEvent = true;
+                        break;
+                }
             }
         }
         
@@ -535,8 +581,14 @@ namespace Smuxi.Frontend.Gnome
             _FontDescription = fontDescription;
             
             _OutputTextView.ModifyFont(_FontDescription);
+            
+            string wrapModeStr = (string) config["Interface/Chat/WrapMode"];
+            if (!String.IsNullOrEmpty(wrapModeStr)) {
+                Gtk.WrapMode wrapMode = (Gtk.WrapMode) Enum.Parse(typeof(Gtk.WrapMode), wrapModeStr);
+                _OutputTextView.WrapMode = wrapMode;
+            }
         }
-
+        
         public virtual void Close()
         {
             Trace.Call();
@@ -611,8 +663,8 @@ namespace Smuxi.Frontend.Gnome
             Gtk.TextIter start = Gtk.TextIter.Zero;
             Gtk.TextIter end = Gtk.TextIter.Zero;
 
-            // if something is selected, bail out
-            if (HasSelection) {
+            // if something in the textview is selected, bail out
+            if (HasTextViewSelection) {
                 return;
             }
             
