@@ -1,9 +1,9 @@
 /*
- * $Id: GroupChatView.cs 334 2008-07-27 19:16:17Z meebey $
+ * $Id: GroupChatView.cs 448 2008-12-22 22:10:06Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Chats/GroupChatView.cs $
- * $Rev: 334 $
+ * $Rev: 448 $
  * $Author: meebey $
- * $Date: 2008-07-27 21:16:17 +0200 (Sun, 27 Jul 2008) $
+ * $Date: 2008-12-22 23:10:06 +0100 (Mon, 22 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -129,9 +129,13 @@ namespace Smuxi.Frontend.Gnome
             
             tv.Model = liststore;
             tv.RowActivated += new Gtk.RowActivatedHandler(OnPersonsRowActivated);
-           
+            tv.FocusOutEvent += OnPersonTreeViewFocusOutEvent;
+            
             // popup menu
             _PersonMenu = new Gtk.Menu();
+            // don't loose the focus else we lose the selection too!
+            // see OnPersonTreeViewFocusOutEvent()
+            _PersonMenu.TakeFocus = false;
             
             _PersonTreeView.ButtonPressEvent += _OnPersonTreeViewButtonPressEvent;
             // frame needed for events when selecting something in the treeview
@@ -426,47 +430,6 @@ namespace Smuxi.Frontend.Gnome
             (cellr as Gtk.CellRendererText).Text = person.IdentityName;
         }
        
-        /*
-        private static int _OnStatusSort(Gtk.TreeModel model, Gtk.TreeIter itera, Gtk.TreeIter iterb)
-        {
-            //Trace.Call(model, itera, iterb);
-            
-            Gtk.ListStore liststore = (Gtk.ListStore)model;
-            // status
-            int    status1a   = 0;
-            string column1a = (string)liststore.GetValue(itera, 0);
-            int    status1b   = 0;
-            string column1b = (string)liststore.GetValue(iterb, 0);
-            // nickname
-            string column2a = (string)liststore.GetValue(itera, 1);
-            string column2b = (string)liststore.GetValue(iterb, 1);
-        
-            if (column1a.IndexOf("@") != -1) {
-                status1a += 1;
-            }
-            if (column1a.IndexOf("+") != -1) {
-                status1a += 2;
-            }
-            if (status1a == 0) {
-                status1a = 4;
-            }
-            column2a = status1a+column2a;
-        
-            if (column1b.IndexOf("@") != -1) {
-                status1b += 1;
-            }
-            if (column1b.IndexOf("+") != -1) {
-                status1b += 2;
-            }
-            if (status1b == 0) {
-                status1b = 4;
-            }
-            column2b = status1b+column2b;
-            
-            return String.Compare(column2a, column2b, true, CultureInfo.InvariantCulture);
-        }
-        */
-        
         protected virtual int SortPersonListStore(Gtk.TreeModel model,
                                                   Gtk.TreeIter iter1,
                                                   Gtk.TreeIter iter2)
@@ -485,6 +448,14 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
         }
         
+        protected virtual void OnPersonTreeViewFocusOutEvent(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+
+            // clear the selection when we loose the focus
+            _PersonTreeView.Selection.UnselectAll();
+        }
+        
         private void _OnUserListButtonReleaseEvent(object sender, Gtk.ButtonReleaseEventArgs e)
         {
             Trace.Call(sender, e);
@@ -500,7 +471,9 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(sender, e);
             
-            // if there is an existing selection prevent making a new one using the right mouse button 
+            // If there is an existing selection prevent making a new one using
+            // the right mouse button.
+            // We have to check > 1 though, because you can't undo a single row selection!
             if (e.Event.Button == 3 && _PersonTreeView.Selection.CountSelectedRows() > 1) {
                 e.RetVal = true;
             }
