@@ -1,9 +1,9 @@
 /*
- * $Id: EngineManagerDialog.cs 384 2008-08-25 21:18:41Z meebey $
+ * $Id: EngineManagerDialog.cs 448 2008-12-22 22:10:06Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/EngineManagerDialog.cs $
- * $Rev: 384 $
+ * $Rev: 448 $
  * $Author: meebey $
- * $Date: 2008-08-25 23:18:41 +0200 (Mon, 25 Aug 2008) $
+ * $Date: 2008-12-22 23:10:06 +0100 (Mon, 22 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -214,7 +214,15 @@ namespace Smuxi.Frontend.Gnome
 #if LOG4NET
                 _Logger.Error(ex);
 #endif
-
+                // clean-up
+                try {
+                    _EngineManager.Disconnect();
+                } catch (Exception disEx) {
+#if LOG4NET
+                    _Logger.Error(disEx);
+#endif
+                }
+                
                 string error_msg = ex.Message + "\n";
                 if (ex.InnerException != null) {
                     error_msg += " [" + ex.InnerException.Message + "]\n";
