@@ -1,9 +1,9 @@
 /*
- * $Id: Entry.cs 381 2008-08-24 18:47:49Z meebey $
+ * $Id: Entry.cs 432 2008-10-07 23:50:16Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Entry.cs $
- * $Rev: 381 $
+ * $Rev: 432 $
  * $Author: meebey $
- * $Date: 2008-08-24 20:47:49 +0200 (Sun, 24 Aug 2008) $
+ * $Date: 2008-10-08 01:50:16 +0200 (Wed, 08 Oct 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -207,6 +207,8 @@ namespace Smuxi.Frontend.Gnome
                             e.RetVal = false;
                         }
                         break;
+                    // don't break unicode input
+                    case Gdk.Key.U:
                     // don't break copy/paste
                     case Gdk.Key.c:
                     case Gdk.Key.v:
