@@ -1,13 +1,13 @@
 /*
- * $Id: Frontend.cs 384 2008-08-25 21:18:41Z meebey $
+ * $Id: Frontend.cs 462 2008-12-28 22:01:24Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Frontend.cs $
- * $Rev: 384 $
+ * $Rev: 462 $
  * $Author: meebey $
- * $Date: 2008-08-25 23:18:41 +0200 (Mon, 25 Aug 2008) $
+ * $Date: 2008-12-28 23:01:24 +0100 (Sun, 28 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2008 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -48,6 +48,7 @@ namespace Smuxi.Frontend.Gnome
         private static readonly string    _UIName = "GTK+";
 #elif UI_GNOME
         private static readonly string    _UIName = "GNOME";
+        private static int                _UIThreadID;
         private static GNOME.Program      _Program;
 #endif
         private static Version            _Version;
@@ -56,6 +57,9 @@ namespace Smuxi.Frontend.Gnome
         private static Version            _EngineVersion;
         private static SplashScreenWindow _SplashScreenWindow;
         private static MainWindow         _MainWindow;
+#if GTK_SHARP_2_10
+        private static Gtk.StatusIcon     _StatusIcon;
+#endif
         private static FrontendConfig     _FrontendConfig;
         private static Session            _LocalSession;
         private static Session            _Session;
@@ -115,7 +119,15 @@ namespace Smuxi.Frontend.Gnome
                 return _MainWindow;
             }
         }
-    
+        
+#if GTK_SHARP_2_10
+        public static Gtk.StatusIcon StatusIcon {
+            get {
+                return _StatusIcon;
+            }
+        }
+#endif
+        
         public static Session Session {
             get {
                 return _Session;
@@ -172,6 +184,7 @@ namespace Smuxi.Frontend.Gnome
             if (!GLib.Thread.Supported) {
                 GLib.Thread.Init();
             }
+            _UIThreadID = System.Threading.Thread.CurrentThread.ManagedThreadId;
 #else
             // with GTK# 2.8 we can do this better, see above
             // GTK# 2.7.1 for MS .NET doesn't support that though.
@@ -196,7 +209,7 @@ namespace Smuxi.Frontend.Gnome
 #if UI_GNOME
             _Program = new GNOME.Program(Name, Version.ToString(), GNOME.Modules.UI, args);
 #elif UI_GTK
-            Gtk.Application.Init();
+            Gtk.Application.Init(Name, args);
 #endif
 #if GTK_SHARP_2_10
             GLib.ExceptionManager.UnhandledException += _OnUnhandledException;
@@ -207,9 +220,30 @@ namespace Smuxi.Frontend.Gnome
             // loading and setting defaults
             _FrontendConfig.Load();
             _FrontendConfig.Save();
+            
+            Gtk.Window.DefaultIcon = new Gdk.Pixbuf(null, "icon.svg");
  
             _MainWindow = new MainWindow();
 
+#if GTK_SHARP_2_10
+            _StatusIcon = new Gtk.StatusIcon();
+            _StatusIcon.Visible = false;
+            _StatusIcon.Pixbuf = new Gdk.Pixbuf(null, "icon.svg");
+            _StatusIcon.Activate += delegate {
+                try {
+                    if (_StatusIcon.Blinking) {
+                        _MainWindow.Present();
+                        return;
+                    }
+                    _MainWindow.Visible = !_MainWindow.Visible;
+                } catch (Exception ex) {
+                    ShowException(ex);
+                }
+            };
+            _StatusIcon.PopupMenu += OnStatusIconPopupMenu;
+            _StatusIcon.Tooltip = "Smuxi";
+#endif
+            
             if (String.IsNullOrEmpty((string) FrontendConfig["Engines/Default"])) {
                 InitLocalEngine();
                 ConnectEngineToGUI();
@@ -223,6 +257,7 @@ namespace Smuxi.Frontend.Gnome
             if (_SplashScreenWindow != null) {
                 _SplashScreenWindow.Destroy();
             }
+            
 #if UI_GNOME
             _Program.Run();
     #if LOG4NET
@@ -256,18 +291,7 @@ namespace Smuxi.Frontend.Gnome
             _FrontendManager = _Session.GetFrontendManager(_MainWindow.UI);
             _FrontendManager.Sync();
             
-            /*
-            // clear all current highlights, annoying to see them each time you
-            // connect to the engine
-            // BUG: this is useless here, the Sync() call triggers only a queue, so
-            // at this point we don't have any chat views yet :(
-            // this was an attempt to fix trac bug #50
-            foreach (ChatView chat in _MainWindow.ChatViewManager.Chats) {
-                chat.HasHighlight = false;
-            }
-            */
-            
-            // MS .NET doesn't like this?
+            // MS .NET doesn't like this with Remoting?
             if (Type.GetType("Mono.Runtime") != null) {
                 // when are running on Mono, all should be good
                 if (_UserConfig.IsCaching) {
@@ -276,8 +300,9 @@ namespace Smuxi.Frontend.Gnome
                 }
             }
             
+            ApplyConfig(_UserConfig);
+            
             _MainWindow.ShowAll();
-            _MainWindow.ApplyConfig(_UserConfig);
             // make sure entry got attention :-P
             _MainWindow.Entry.HasFocus = true;
             
@@ -300,24 +325,38 @@ namespace Smuxi.Frontend.Gnome
         
         public static void Quit()
         {
-            // save window size
-            int width, heigth;
-            _MainWindow.GetSize(out width, out heigth);
-            _FrontendConfig[Frontend.UIName + "/Interface/Width"] = width;
-            _FrontendConfig[Frontend.UIName + "/Interface/Heigth"] = heigth;
-            
-            int x, y;
-            _MainWindow.GetPosition(out x, out y);
-            _FrontendConfig[Frontend.UIName + "/Interface/XPosition"] = x;
-            _FrontendConfig[Frontend.UIName + "/Interface/YPosition"] = y;
-            _FrontendConfig.Save();
+            Trace.Call();
+
+            // only save windows size when we are not in the engine manager dialog
+            if (_MainWindow.Visible) {
+                // save window size
+                int width, heigth;
+                if (_MainWindow.IsMaximized) {
+                    width = -1;
+                    heigth = -1;
+                } else {
+                    _MainWindow.GetSize(out width, out heigth);
+                }
+                _FrontendConfig[Frontend.UIName + "/Interface/Width"] = width;
+                _FrontendConfig[Frontend.UIName + "/Interface/Heigth"] = heigth;
+                
+                int x, y;
+                _MainWindow.GetPosition(out x, out y);
+                _FrontendConfig[Frontend.UIName + "/Interface/XPosition"] = x;
+                _FrontendConfig[Frontend.UIName + "/Interface/YPosition"] = y;
+                _FrontendConfig.Save();
+            }
             
             if (_FrontendManager != null) {
-                _FrontendManager.IsFrontendDisconnecting = true;
+                try {
+                    _FrontendManager.IsFrontendDisconnecting = true;
+                } catch (System.Runtime.Remoting.RemotingException ex) {
+                    // the connection is maybe already gone
+                }
             }
             
             /*
-            // BUG: don't do this, the access to config is lost and the entry will
+            // WASBUG: don't do this, the access to config is lost and the entry will
             // throw an exception then.
             if (_FrontendManager != null) {
                 DisconnectEngineFromGUI();
@@ -333,17 +372,83 @@ namespace Smuxi.Frontend.Gnome
             Environment.Exit(0);
         }
         
-        public static void ShowError(string msg)
+        private static bool IsGuiThread()
         {
-            Gtk.MessageDialog md = new Gtk.MessageDialog(_MainWindow,
-                Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
-                Gtk.ButtonsType.Ok, msg);
+            return System.Threading.Thread.CurrentThread.ManagedThreadId == _UIThreadID;
+        }
+                
+        public static void ShowError(Gtk.Window parent, string msg, Exception ex)
+        {
+            Trace.Call(parent, msg, ex != null ? ex.GetType() : null);
+            
+            if (!IsGuiThread()) {
+                Gtk.Application.Invoke(delegate {
+                    ShowError(parent, msg, ex);
+                });
+                return;
+            }
+            
+            if (ex != null) {
+                msg += "\n" + String.Format(_("Cause: {0}"), ex.Message);
+            }
+            if (parent == null) {
+                parent = _MainWindow;
+            }
+            
+            Gtk.MessageDialog md = new Gtk.MessageDialog(
+                parent,
+                Gtk.DialogFlags.Modal,
+                Gtk.MessageType.Error,
+                Gtk.ButtonsType.Ok,
+                msg
+            );
             md.Run();
             md.Destroy();
         }
         
+        public static void ShowError(Gtk.Window parent, string msg)
+        {
+            Trace.Call(parent, msg);
+            
+            ShowError(parent, msg, null);
+        }
+        
         public static void ShowException(Gtk.Window parent, Exception ex)
         {
+            Trace.Call(parent, ex != null ? ex.GetType() : null);
+            
+            if (parent == null) {
+                parent = _MainWindow;
+            }
+            
+            if (!IsGuiThread()) {
+                Gtk.Application.Invoke(delegate {
+                    ShowException(parent, ex);
+                });
+                return;
+            }
+            
+#if LOG4NET
+            _Logger.Error("ShowException(): Exception:", ex);
+#endif
+            
+            if (ex is System.Runtime.Remoting.RemotingException) {
+                Gtk.MessageDialog md = new Gtk.MessageDialog(parent,
+                    Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
+                    Gtk.ButtonsType.OkCancel, _("The frontend has lost the connection to the server.\n Do you want to reconnect now?"));
+                Gtk.ResponseType res = (Gtk.ResponseType) md.Run();
+                md.Destroy();
+                
+                if (res == Gtk.ResponseType.Ok) {
+                    Frontend.DisconnectEngineFromGUI();
+                    _MainWindow.EngineManager.Reconnect();
+                    return;
+                }
+                
+                Quit();
+                return;
+            }
+            
             CrashDialog cd = new CrashDialog(parent, ex);
             cd.Run();
             cd.Destroy();
@@ -353,6 +458,8 @@ namespace Smuxi.Frontend.Gnome
         
         public static void ShowException(Exception ex)
         {
+            Trace.Call(ex != null ? ex.GetType() : null);
+            
             ShowException(null, ex);
         }
         
@@ -365,6 +472,72 @@ namespace Smuxi.Frontend.Gnome
             diag.Destroy();
         }
         
+        public static void ApplyConfig(UserConfig userConfig)
+        {
+            Trace.Call(userConfig);
+            
+            if (userConfig == null) {
+                throw new ArgumentNullException("userConfig");
+            }
+            
+#if GTK_SHARP_2_10
+            string modeStr = (string) userConfig["Interface/Notification/NotificationAreaIconMode"];
+            NotificationAreaIconMode mode = (NotificationAreaIconMode) Enum.Parse(
+                typeof(NotificationAreaIconMode),
+                modeStr
+            );
+            switch (mode) {
+                case NotificationAreaIconMode.Never:
+                    _StatusIcon.Visible = false;
+                    break;
+                case NotificationAreaIconMode.Always:
+                    _StatusIcon.Visible = true;
+                    break;
+                case NotificationAreaIconMode.Minimized:
+                case NotificationAreaIconMode.Closed:
+                    // at application startup the main window is not realized but not visible
+                    _StatusIcon.Visible = _MainWindow.IsRealized && !_MainWindow.Visible;
+                    break;
+            }
+#endif
+            
+            _MainWindow.ApplyConfig(userConfig);
+        }
+        
+        private static void OnStatusIconPopupMenu(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            Gtk.Menu menu = new Gtk.Menu();
+            
+            Gtk.ImageMenuItem preferencesItem = new Gtk.ImageMenuItem(Gtk.Stock.Preferences, null);
+            preferencesItem.Activated += delegate {
+                try {
+                    PreferencesDialog dialog = new PreferencesDialog(_MainWindow);
+                    dialog.CurrentPage = PreferencesDialog.Page.Interface;
+                    dialog.CurrentInterfacePage = PreferencesDialog.InterfacePage.Notification;
+                } catch (Exception ex) {
+                    ShowException(ex);
+                }
+            };
+            menu.Add(preferencesItem);
+            
+            menu.Add(new Gtk.SeparatorMenuItem());
+            
+            Gtk.ImageMenuItem quitItem = new Gtk.ImageMenuItem(Gtk.Stock.Quit, null);
+            quitItem.Activated += delegate {
+                try {
+                    Quit();
+                } catch (Exception ex) {
+                    ShowException(ex);
+                }
+            };
+            menu.Add(quitItem);
+            
+            menu.ShowAll();
+            menu.Popup();
+        }
+        
 #if GTK_SHARP_2_10
         private static void _OnUnhandledException(GLib.UnhandledExceptionArgs e)
         {
