@@ -1,9 +1,9 @@
 /*
- * $Id: MainWindow.cs 381 2008-08-24 18:47:49Z meebey $
+ * $Id: MainWindow.cs 478 2008-12-31 18:17:56Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/MainWindow.cs $
- * $Rev: 381 $
+ * $Rev: 478 $
  * $Author: meebey $
- * $Date: 2008-08-24 20:47:49 +0200 (Sun, 24 Aug 2008) $
+ * $Date: 2008-12-31 19:17:56 +0100 (Wed, 31 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -62,6 +62,8 @@ namespace Smuxi.Frontend.Gnome
         private IFrontendUI      _UI;
         private EngineManager    _EngineManager;
         private Gtk.MenuItem     _CloseChatMenuItem;
+        private NotificationAreaIconMode _NotificationAreaIconMode;
+        private bool             _IsMaximized;
         
         public bool CaretMode {
             get {
@@ -124,7 +126,13 @@ namespace Smuxi.Frontend.Gnome
                 return _Entry;
             }
         }
-     
+        
+        public bool IsMaximized {
+            get {
+                return _IsMaximized;
+            }
+        }
+        
 #if UI_GNOME
         public MainWindow() : base("smuxi", "smuxi - Smart MUtipleXed Irc")
 #elif UI_GTK
@@ -143,7 +151,12 @@ namespace Smuxi.Frontend.Gnome
             } else {
                 heigth = 600;
             }
-            SetDefaultSize(width, heigth);
+            if (width == -1 && heigth == -1) {
+                SetDefaultSize(800, 600);
+                Maximize();
+            } else {
+                SetDefaultSize(width, heigth);
+            }
             
             int x, y;
             if (Frontend.FrontendConfig[Frontend.UIName + "/Interface/XPosition"] != null) {
@@ -162,8 +175,11 @@ namespace Smuxi.Frontend.Gnome
                 Move(x, y);
             }
             
-            Destroyed += new EventHandler(_OnDestroyed);
-            FocusInEvent += new Gtk.FocusInEventHandler(_OnFocusInEvent);
+            DeleteEvent += OnDeleteEvent;
+            FocusInEvent += OnFocusInEvent;
+            WindowStateEvent += OnWindowStateEvent;
+                    
+            
             
             Gtk.AccelGroup agrp = new Gtk.AccelGroup();
             Gtk.AccelKey   akey;
@@ -258,6 +274,30 @@ namespace Smuxi.Frontend.Gnome
             menu.Append(new Gtk.SeparatorMenuItem());
             */
             
+            /*
+            image_item = new Gtk.ImageMenuItem(Gtk.Stock.Find, agrp);
+            image_item.Activated += OnFindChatMenuItemActivated;
+            menu.Append(image_item);
+            
+            item = new Gtk.MenuItem(_("Find _Next"));
+            item.Activated += OnFindNextChatMenuItemActivated;
+            akey = new Gtk.AccelKey();
+            akey.AccelFlags = Gtk.AccelFlags.Visible;
+            akey.AccelMods = Gdk.ModifierType.ControlMask;
+            akey.Key = Gdk.Key.G;
+            item.AddAccelerator("activate", agrp, akey);
+            menu.Append(item);
+            
+            item = new Gtk.MenuItem(_("Find _Previous"));
+            item.Activated += OnFindPreviousChatMenuItemActivated;
+            akey = new Gtk.AccelKey();
+            akey.AccelFlags = Gtk.AccelFlags.Visible;
+            akey.AccelMods = Gdk.ModifierType.ControlMask | Gdk.ModifierType.ShiftMask;
+            akey.Key = Gdk.Key.G;
+            item.AddAccelerator("activate", agrp, akey);
+            menu.Append(item);
+            */
+            
             _CloseChatMenuItem = new Gtk.ImageMenuItem(Gtk.Stock.Close, agrp);
             _CloseChatMenuItem.Activated += OnCloseChatMenuItemActivated;
             menu.Append(_CloseChatMenuItem);
@@ -374,6 +414,13 @@ namespace Smuxi.Frontend.Gnome
                 throw new ArgumentNullException("userConfig");
             }
                     
+            string modeStr = (string) userConfig["Interface/Notification/NotificationAreaIconMode"];
+            NotificationAreaIconMode mode = (NotificationAreaIconMode) Enum.Parse(
+                typeof(NotificationAreaIconMode),
+                modeStr
+            );
+            _NotificationAreaIconMode = mode;
+            
             _Entry.ApplyConfig(userConfig);
             _Notebook.ApplyConfig(userConfig);
             _ChatViewManager.ApplyConfig(userConfig);
@@ -390,23 +437,36 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
-        private void _OnDestroyed(object obj, EventArgs args)
+        protected virtual void OnDeleteEvent(object sender, Gtk.DeleteEventArgs e)
         {
-            Trace.Call(obj, args);
+            Trace.Call(sender, e);
             
             try {
+                if (_NotificationAreaIconMode == NotificationAreaIconMode.Closed) {
+                    // showing the tray icon is handled in OnWindowStateEvent
+                    Hide();
+                    
+                    // don't destroy the window nor quit smuxi!
+                    e.RetVal = true;
+                    return;
+                }
+                
                 Frontend.Quit();
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
         }
     
-        private void _OnFocusInEvent(object obj, EventArgs args)
+        protected virtual void OnFocusInEvent(object sender, EventArgs e)
         {
-            Trace.Call(obj, args);
+            Trace.Call(sender, e);
             
             try {
                 UrgencyHint = false;
+#if GTK_SHARP_2_10
+                // HACK: out of scope?
+                Frontend.StatusIcon.Blinking = false;
+#endif
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -417,7 +477,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             try {
-                QuickConnectDialog dialog = new QuickConnectDialog();
+                QuickConnectDialog dialog = new QuickConnectDialog(this);
                 dialog.Load();
                 int res = dialog.Run();
                 ServerModel server = dialog.Server;
@@ -432,32 +492,28 @@ namespace Smuxi.Frontend.Gnome
 #endif
                 }
                 
-                CommandModel cmd = new CommandModel(
-                    Frontend.FrontendManager,
-                    Frontend.Session.SessionChat,
-                    "/",
-                    String.Format(
-                        "/connect {0} {1} {2} {3}",
+                Frontend.Session.Connect(
+                        Frontend.FrontendManager,
                         server.Protocol,
                         server.Hostname,
                         server.Port,
                         server.Username,
                         server.Password
-                    )
                 );
-                Frontend.Session.CommandConnect(cmd);
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
         }
 
+#region Item Event Handlers
         protected virtual void OnServerAddButtonClicked(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
             
             try {
                 ServerListController controller = new ServerListController(Frontend.UserConfig);
-                ServerView serverView = new ServerView(null,
+                ServerView serverView = new ServerView(this,
+                                                       null,
                                                        Frontend.Session.GetSupportedProtocols(),
                                                        controller.GetNetworks());
                 int res = serverView.Run();
@@ -478,7 +534,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             try {
-                PreferencesDialog dialog = new PreferencesDialog();
+                PreferencesDialog dialog = new PreferencesDialog(this);
                 dialog.CurrentPage = PreferencesDialog.Page.Servers;
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
@@ -507,6 +563,36 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
+        protected virtual void OnFindChatMenuItemActivated(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            try {
+            } catch (Exception ex) {
+                Frontend.ShowException(this, ex);
+            }
+        }
+        
+        protected virtual void OnFindNextChatMenuItemActivated(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            try {
+            } catch (Exception ex) {
+                Frontend.ShowException(this, ex);
+            }
+        }
+        
+        protected virtual void OnFindPreviousChatMenuItemActivated(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            try {
+            } catch (Exception ex) {
+                Frontend.ShowException(this, ex);
+            }
+        }
+        
         protected virtual void OnCloseChatMenuItemActivated(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
@@ -543,6 +629,48 @@ namespace Smuxi.Frontend.Gnome
                 Frontend.ShowException(this, ex);
             }
         }
+                
+        protected virtual void OnWindowStateEvent(object sender, Gtk.WindowStateEventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            try {
+#if GTK_SHARP_2_10
+                // handle minimize / un-minimize
+                if (_NotificationAreaIconMode == NotificationAreaIconMode.Minimized &&
+                    (e.Event.ChangedMask & Gdk.WindowState.Iconified) != 0) {
+                    bool isMinimized = (e.Event.NewWindowState & Gdk.WindowState.Iconified) != 0;
+    #if LOG4NET
+                    f_Logger.Debug("OnWindowStateEvent(): isMinimized: " + isMinimized);
+    #endif
+                    Frontend.StatusIcon.Visible = isMinimized;
+                    if (isMinimized) {
+                        Hide();
+                    }
+                }
+
+                // handle hide / show
+                if (_NotificationAreaIconMode == NotificationAreaIconMode.Closed &&
+                    (e.Event.ChangedMask & Gdk.WindowState.Withdrawn) != 0) {
+                    bool isHidden = (e.Event.NewWindowState & Gdk.WindowState.Withdrawn) != 0;
+    #if LOG4NET
+                    f_Logger.Debug("OnWindowStateEvent(): isHidden: " + isHidden);
+    #endif
+                    Frontend.StatusIcon.Visible = isHidden;
+                }
+#endif
+                
+                // handle maximize / un-maximize
+                if ((e.Event.ChangedMask & Gdk.WindowState.Maximized) != 0) {
+                    _IsMaximized = (e.Event.NewWindowState & Gdk.WindowState.Maximized) != 0;
+#if LOG4NET
+                    f_Logger.Debug("OnWindowStateEvent(): _IsMaximized: " + _IsMaximized);
+#endif
+                }
+            } catch (Exception ex) {
+                Frontend.ShowException(this, ex);
+            }
+        }
         
         protected virtual void OnNotebookSwitchPage(object sender, EventArgs e)
         {
@@ -560,7 +688,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(obj, args);
             
             try {
-                AboutDialog ad = new AboutDialog();
+                AboutDialog ad = new AboutDialog(this);
                 ad.Run();
                 ad.Destroy();
             } catch (Exception ex) {
@@ -573,7 +701,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(obj, args);
             
             try {
-                new PreferencesDialog();
+                new PreferencesDialog(this);
                 /*
                 SteticPreferencesDialog dialog = new SteticPreferencesDialog();
                 dialog.Run();
@@ -659,6 +787,7 @@ namespace Smuxi.Frontend.Gnome
                 Frontend.ShowException(this, ex);
             }
         }
+#endregion
         
         private static string _(string msg)
         {
