@@ -1,9 +1,9 @@
 /*
- * $Id: Notebook.cs 384 2008-08-25 21:18:41Z meebey $
+ * $Id: Notebook.cs 458 2008-12-28 21:13:04Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Notebook.cs $
- * $Rev: 384 $
+ * $Rev: 458 $
  * $Author: meebey $
- * $Date: 2008-08-25 23:18:41 +0200 (Mon, 25 Aug 2008) $
+ * $Date: 2008-12-28 22:13:04 +0100 (Sun, 28 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -132,6 +132,10 @@ namespace Smuxi.Frontend.Gnome
                     // even when we have no network manager, we still want to update the state
                     Frontend.FrontendManager.UpdateNetworkStatus();
 
+                    // update last seen highlight 
+                    if (chatView.HasHighlight) {
+                        chatModel.LastSeenHighlight = DateTime.UtcNow;
+                    }
                     // clear activity and highlight
                     chatView.HasHighlight = false;
                     chatView.HasActivity = false;
@@ -142,9 +146,6 @@ namespace Smuxi.Frontend.Gnome
                         Frontend.MainWindow.Title = network + chatView.Name +
                                                     " - smuxi - Smart MUtipleXed Irc";
                     }
-                    
-                    // switching pages might grab the focus, so let's move it back to the entry
-                    //Frontend.MainWindow.Entry.HasFocus = true;
                 }
             } catch (Exception ex) {
                 Frontend.ShowException(null, ex);
