@@ -1,9 +1,9 @@
 /*
- * $Id: PreferencesDialog.cs 368 2008-08-21 18:54:44Z meebey $
+ * $Id: PreferencesDialog.cs 460 2008-12-28 22:01:01Z meebey $
  * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Preferences/PreferencesDialog.cs $
- * $Rev: 368 $
+ * $Rev: 460 $
  * $Author: meebey $
- * $Date: 2008-08-21 20:54:44 +0200 (Thu, 21 Aug 2008) $
+ * $Date: 2008-12-28 23:01:01 +0100 (Sun, 28 Dec 2008) $
  *
  * smuxi - Smart MUltipleXed Irc
  *
@@ -32,6 +32,7 @@ using System.Collections;
 using System.Globalization;
 using Smuxi;
 using Smuxi.Common;
+using Smuxi.Engine;
 
 namespace Smuxi.Frontend.Gnome
 {
@@ -44,6 +45,14 @@ namespace Smuxi.Frontend.Gnome
             Filters,
         }
         
+        public enum InterfacePage : int {
+            General = 0,
+            Tabs,
+            Input,
+            Output,
+            Notification
+        }
+        
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
@@ -53,6 +62,8 @@ namespace Smuxi.Frontend.Gnome
 #region Widgets
         [Glade.Widget("Notebook")]
         private Gtk.Notebook _Notebook;
+        [Glade.Widget("InterfaceNotebook")]
+        private Gtk.Notebook _InterfaceNotebook;
         [Glade.Widget("MenuTreeView")]
         private Gtk.TreeView _MenuTreeView;
 #endregion
@@ -77,15 +88,29 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
-        public PreferencesDialog()
+        public InterfacePage CurrentInterfacePage {
+            get {
+                return (InterfacePage) _InterfaceNotebook.CurrentPage;
+            }
+            set {
+                _InterfaceNotebook.CurrentPage = (int) value;
+            }
+        }
+        
+        public PreferencesDialog(Gtk.Window parent)
         {
-            Trace.Call();
+            Trace.Call(parent);
 
+            if (parent == null) {
+                throw new ArgumentNullException("parent");
+            }
+            
             _Glade = new Glade.XML(null, Frontend.GladeFilename, "PreferencesDialog", null);
             //_Glade.BindFields(this);
             // changed signal is used in all settings, so use glade for now
             _Glade.Autoconnect(this);
             _Dialog = (Gtk.Dialog)_Glade["PreferencesDialog"];
+            _Dialog.TransientFor = parent;
             
             ((Gtk.Button)_Glade["OKButton"]).Clicked += new EventHandler(_OnOKButtonClicked);
             ((Gtk.Button)_Glade["ApplyButton"]).Clicked += new EventHandler(_OnApplyButtonClicked);
@@ -94,9 +119,30 @@ namespace Smuxi.Frontend.Gnome
             ((Gtk.TextView)_Glade["OnConnectCommandsTextView"]).Buffer.Changed += new EventHandler(_OnChanged);
             ((Gtk.TextView)_Glade["OnStartupCommandsTextView"]).Buffer.Changed += new EventHandler(_OnChanged);
             
+            ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Toggled += OnNotificationAreaIconCheckButtonToggled;
+            ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Toggled += _OnChanged;
+            ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonAlways"]).Toggled += _OnChanged;
+            ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonMinimized"]).Toggled += _OnChanged;
+            ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonClosed"]).Toggled += _OnChanged;
+            
             ((Gtk.CheckButton)_Glade["OverrideForegroundColorCheckButton"]).Toggled += OnOverrideForegroundColorCheckButtonToggled;
             ((Gtk.CheckButton)_Glade["OverrideBackgroundColorCheckButton"]).Toggled += OnOverrideBackgroundColorCheckButtonToggled;
             ((Gtk.CheckButton)_Glade["OverrideFontCheckButton"]).Toggled += OnOverrideFontCheckButtonToggled;
+
+            Gtk.ComboBox wrapModeComboBox = (Gtk.ComboBox)_Glade["WrapModeComboBox"];
+            // initialize wrap modes
+            // glade might initialize it already!
+            wrapModeComboBox.Clear();
+            wrapModeComboBox.Changed += _OnChanged;
+            Gtk.CellRendererText cell = new Gtk.CellRendererText();
+            wrapModeComboBox.PackStart(cell, false);
+            wrapModeComboBox.AddAttribute(cell, "text", 1);
+            Gtk.ListStore store = new Gtk.ListStore(typeof(Gtk.WrapMode), typeof(string));
+            // fill ListStore
+            store.AppendValues(Gtk.WrapMode.Char,     _("Character"));
+            store.AppendValues(Gtk.WrapMode.WordChar, _("Word"));
+            wrapModeComboBox.Model = store;
+            wrapModeComboBox.Active = 0;
             
             _Notebook.ShowTabs = false;
             
@@ -113,10 +159,12 @@ namespace Smuxi.Frontend.Gnome
                                                 Gtk.Stock.Network,
                                                 Gtk.IconSize.SmallToolbar, null),
                             _("Servers"));
+            /*
             ls.AppendValues(Page.Filters, _Dialog.RenderIcon(
                                                 Gtk.Stock.Delete,
                                                 Gtk.IconSize.SmallToolbar, null),
                             _("Filters"));
+            */
             
             int i = 1;
             _MenuTreeView.AppendColumn(null, new Gtk.CellRendererPixbuf(), "pixbuf",i++);
@@ -131,7 +179,7 @@ namespace Smuxi.Frontend.Gnome
             _MenuTreeView.Selection.SelectIter(iter);
             
             _ChannelFilterListView = new ChannelFilterListView(_Glade);
-            _ServerListView = new ServerListView(_Glade);
+            _ServerListView = new ServerListView(parent, _Glade);
             
             _Load();
         }
@@ -277,7 +325,7 @@ namespace Smuxi.Frontend.Gnome
             colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
 
             colorButton = (Gtk.ColorButton)_Glade["ModeColorButton"];
-            colorHexCode = (string)Frontend.UserConfig["Interface/Notebook/Tab/ModeColor"];
+            colorHexCode = (string)Frontend.UserConfig["Interface/Notebook/Tab/EventColor"];
             colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
             
             colorButton = (Gtk.ColorButton)_Glade["HighlightColorButton"];
@@ -332,6 +380,20 @@ namespace Smuxi.Frontend.Gnome
                 fontButton.FontName = fontDescription.ToString();
             }
             
+            Gtk.ComboBox wrapModeComboBox = ((Gtk.ComboBox)_Glade["WrapModeComboBox"]);
+            Gtk.WrapMode wrapMode = (Gtk.WrapMode) Enum.Parse(
+                typeof(Gtk.WrapMode),
+                (string) Frontend.UserConfig["Interface/Chat/WrapMode"]
+            );
+            int i = 0;
+            foreach (object[] row in  (Gtk.ListStore) wrapModeComboBox.Model) {
+                if (((Gtk.WrapMode) row[0]) == wrapMode) {
+                    wrapModeComboBox.Active = i;
+                    break;
+                }
+                i++;
+            }
+            
             // Interface/Entry
             ((Gtk.Entry)_Glade["CompletionCharacterEntry"]).Text =
                 (string)Frontend.UserConfig["Interface/Entry/CompletionCharacter"];
@@ -345,6 +407,31 @@ namespace Smuxi.Frontend.Gnome
             ((Gtk.CheckButton)_Glade["BeepOnHighlightCheckButton"]).Active =
                 (bool)Frontend.UserConfig["Sound/BeepOnHighlight"];
             
+            // Interface/Notification
+            string modeStr = (string) Frontend.UserConfig["Interface/Notification/NotificationAreaIconMode"];
+            NotificationAreaIconMode mode = (NotificationAreaIconMode) Enum.Parse(
+                typeof(NotificationAreaIconMode),
+                modeStr
+            );
+            switch (mode) {
+                case NotificationAreaIconMode.Never:
+                    ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Active = false;
+                    ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonMinimized"]).Active = true;
+                    break;
+                case NotificationAreaIconMode.Always:
+                    ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Active = true;
+                    ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonAlways"]).Active = true;
+                    break;
+                case NotificationAreaIconMode.Minimized:
+                    ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Active = true;
+                    ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonMinimized"]).Active = true;
+                    break;
+                case NotificationAreaIconMode.Closed:
+                    ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Active = true;
+                    ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonClosed"]).Active = true;
+                    break;
+            }
+            
             // Filters
             _ChannelFilterListView.Load();
             
@@ -358,6 +445,10 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
             
+            if (((Gtk.Entry)_Glade["ConnectionNicknamesEntry"]).Text.Trim().Length == 0) {
+                throw new ApplicationException(_("Nicknames(s) field must not be empty."));
+            }
+            
             string prefix;
             
             // root
@@ -436,7 +527,7 @@ namespace Smuxi.Frontend.Gnome
                 ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["NoActivityColorButton"]).Color);
             Frontend.UserConfig[prefix + "ActivityColor"] =
                 ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["ActivityColorButton"]).Color);
-            Frontend.UserConfig[prefix + "ModeColor"] =
+            Frontend.UserConfig[prefix + "EventColor"] =
                 ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["ModeColorButton"]).Color);
             Frontend.UserConfig[prefix + "HighlightColor"] =
                 ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["HighlightColorButton"]).Color);
@@ -467,6 +558,18 @@ namespace Smuxi.Frontend.Gnome
                 Frontend.UserConfig[prefix + "FontSize"] = 0;
             }
             
+            Gtk.ComboBox wrapModeComboBox = (Gtk.ComboBox) _Glade["WrapModeComboBox"];
+            Gtk.WrapMode wrapMode = Gtk.WrapMode.Char;
+            int i = 0;
+            foreach (object[] row in (Gtk.ListStore) wrapModeComboBox.Model) {
+                if (wrapModeComboBox.Active == i) {
+                    wrapMode = (Gtk.WrapMode) row[0];
+                    break;
+                }
+                i++;
+            }
+            Frontend.UserConfig[prefix + "WrapMode"] = wrapMode.ToString();
+            
             // Entry
             Frontend.UserConfig["Interface/Entry/CompletionCharacter"] =
                 ((Gtk.Entry)_Glade["CompletionCharacterEntry"]).Text;
@@ -480,6 +583,23 @@ namespace Smuxi.Frontend.Gnome
             Frontend.UserConfig["Sound/BeepOnHighlight"] =
                 ((Gtk.CheckButton)_Glade["BeepOnHighlightCheckButton"]).Active;
             
+            // Interface/Notification
+            if (((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Active) {
+                if (((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonAlways"]).Active) {
+                    Frontend.UserConfig["Interface/Notification/NotificationAreaIconMode"] =
+                        NotificationAreaIconMode.Always.ToString();
+                } else if (((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonMinimized"]).Active) {
+                        Frontend.UserConfig["Interface/Notification/NotificationAreaIconMode"] =
+                            NotificationAreaIconMode.Minimized.ToString();
+                } else if (((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonClosed"]).Active) {
+                        Frontend.UserConfig["Interface/Notification/NotificationAreaIconMode"] =
+                            NotificationAreaIconMode.Closed.ToString();
+                }
+            } else {
+                    Frontend.UserConfig["Interface/Notification/NotificationAreaIconMode"] =
+                        NotificationAreaIconMode.Never.ToString();
+            }
+            
             // Filters
             _ChannelFilterListView.Save();
             
@@ -489,7 +609,7 @@ namespace Smuxi.Frontend.Gnome
             Frontend.Config.Save();
         }
         
-        private void _OnChanged(object sender, EventArgs e)
+        protected virtual void _OnChanged(object sender, EventArgs e)
         {
             ((Gtk.Button)_Glade["ApplyButton"]).Sensitive = true;
         }
@@ -502,8 +622,10 @@ namespace Smuxi.Frontend.Gnome
                 _Save();
                 Frontend.Config.Load();
                 Frontend.UserConfig.ClearCache();
-                Frontend.MainWindow.ApplyConfig(Frontend.UserConfig);
+                Frontend.ApplyConfig(Frontend.UserConfig);
                 _Dialog.Destroy();
+            } catch (ApplicationException ex) {
+                Frontend.ShowError(_Dialog, ex.Message);
             } catch (Exception ex) {
 #if LOG4NET
                 _Logger.Error(ex);
@@ -522,7 +644,7 @@ namespace Smuxi.Frontend.Gnome
                 Frontend.Config.Load();
                 Frontend.UserConfig.ClearCache();
                 _Load();
-                Frontend.MainWindow.ApplyConfig(Frontend.UserConfig);
+                Frontend.ApplyConfig(Frontend.UserConfig);
             } catch (Exception ex) {
 #if LOG4NET
                 _Logger.Error(ex);
@@ -593,6 +715,23 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
+        protected virtual void OnNotificationAreaIconCheckButtonToggled(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+            
+            try {
+                bool isActive = ((Gtk.CheckButton) _Glade["NotificationAreaIconCheckButton"]).Active;
+                if (!isActive) {
+                    ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonMinimized"]).Active = true;
+                }
+                ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonAlways"]).Sensitive = isActive;
+                ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonMinimized"]).Sensitive = isActive;
+                ((Gtk.RadioButton) _Glade["NotificationAreaIconRadioButtonClosed"]).Sensitive = isActive;
+            } catch (Exception ex) {
+                Frontend.ShowException(ex);
+            }
+        }
+        
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
